@@ -73,6 +73,7 @@ module Control.Foldl (
     , findIndex
     , random
     , randomN
+    , Control.Foldl.mapM_
     , sink
 
     -- * Generic Folds
@@ -90,9 +91,12 @@ module Control.Foldl (
     -- * Utilities
     -- $utilities
     , purely
+    , purely_
     , impurely
+    , impurely_
     , generalize
     , simplify
+    , hoists
     , duplicateM
     , _Fold1
     , premap
@@ -120,7 +124,7 @@ import Data.Functor.Constant (Constant(Constant, getConstant))
 import Data.Functor.Identity (Identity, runIdentity)
 import Data.Monoid
 import Data.Profunctor
-import Data.Sequence ((<|))
+import Data.Sequence ((|>))
 import Data.Vector.Generic (Vector, Mutable)
 import Data.Vector.Generic.Mutable (MVector)
 import System.Random.MWC (GenIO, createSystemRandom, uniformR)
@@ -481,7 +485,7 @@ lastDef a = Fold (\_ a' -> a') a id
 lastN :: Int -> Fold a [a]
 lastN n = Fold step begin done
   where
-    step s a = a <| s'
+    step s a = s' |> a
       where
         s' =
             if Seq.length s < n
@@ -662,6 +666,11 @@ randomN n = FoldM step begin done
         v <- V.freeze mv
         return (Just v)
 
+-- | Converts an effectful function to a fold. Specialized version of 'sink'.
+mapM_ :: Monad m => (a -> m ()) -> FoldM m a ()
+mapM_ = sink
+{-# INLINABLE mapM_ #-}
+
 {-| Converts an effectful function to a fold
 
 > sink (f <> g) = sink f <> sink g -- if `(<>)` is commutative
@@ -782,6 +791,11 @@ purely :: (forall x . (x -> a -> x) -> x -> (x -> b) -> r) -> Fold a b -> r
 purely f (Fold step begin done) = f step begin done
 {-# INLINABLE purely #-}
 
+-- | Upgrade a more traditional fold to accept the `Fold` type
+purely_ :: (forall x . (x -> a -> x) -> x -> x) -> Fold a b -> b
+purely_ f (Fold step begin done) = done (f step begin)
+{-# INLINABLE purely_ #-}
+
 -- | Upgrade a monadic fold to accept the 'FoldM' type
 impurely
     :: Monad m
@@ -791,6 +805,15 @@ impurely
 impurely f (FoldM step begin done) = f step begin done
 {-# INLINABLE impurely #-}
 
+-- | Upgrade a more traditional monadic fold to accept the `FoldM` type
+impurely_
+    :: Monad m
+    => (forall x . (x -> a -> m x) -> m x -> m x) -> FoldM m a b -> m b
+impurely_ f (FoldM step begin done) = do
+    x <- f step begin
+    done x
+{-# INLINABLE impurely_ #-}
+
 {-| Generalize a `Fold` to a `FoldM`
 
 > generalize (pure r) = pure r
@@ -819,13 +842,20 @@ simplify (FoldM step begin done) = Fold step' begin' done'
     done' x   = runIdentity (done x)
 {-# INLINABLE simplify #-}
 
-{-| Allows to continue feeding a 'FoldM' even after passing it to a function 
+
+{- | Shift a 'FoldM' from one monad to another with a morphism such as 'lift' or 'liftIO';
+     the effect is the same as 'Control.Monad.Morph.hoist'.
+-}
+hoists :: Monad m => (forall x . m x -> n x) -> FoldM m a b -> FoldM n a b
+hoists phi (FoldM step begin done) = FoldM (\a b -> phi (step a b)) (phi begin) (phi . done)
+
+{-| Allows to continue feeding a 'FoldM' even after passing it to a function
 that closes it.
 
 For pure 'Fold's, this is provided by the 'Control.Comonad.Comonad' instance.
 -}
 duplicateM :: Applicative m => FoldM m a b -> FoldM m a (FoldM m a b)
-duplicateM (FoldM step begin done) = 
+duplicateM (FoldM step begin done) =
     FoldM step begin (\x -> pure (FoldM step (pure x) done))
 {-# INLINABLE duplicateM #-}
 
@@ -889,7 +919,7 @@ premapM f (FoldM step begin done) = FoldM step' begin done
     Any lens, traversal, or prism will type-check as a `Handler`
 -}
 type Handler a b =
-    forall x . (b -> Constant (Endo x) b) -> a -> Constant (Endo x) a 
+    forall x . (b -> Constant (Endo x) b) -> a -> Constant (Endo x) a
 
 {-| @(handles t folder)@ transforms the input of a `Fold` using a lens,
     traversal, or prism:
@@ -940,7 +970,7 @@ instance Monad m => Monoid (EndoM m a) where
     Any lens, traversal, or prism will type-check as a `HandlerM`
 -}
 type HandlerM m a b =
-    forall x . (b -> Constant (EndoM m x) b) -> a -> Constant (EndoM m x) a 
+    forall x . (b -> Constant (EndoM m x) b) -> a -> Constant (EndoM m x) a
 
 {-| @(handlesM t folder)@ transforms the input of a `FoldM` using a lens,
     traversal, or prism:
